package de.intevation.flys.client.client.ui.map;

import com.google.gwt.core.client.GWT;
import com.smartgwt.client.widgets.Canvas;
import com.smartgwt.client.widgets.WidgetCanvas;
import com.smartgwt.client.widgets.events.ParentMovedEvent;
import com.smartgwt.client.widgets.events.ParentMovedHandler;
import com.smartgwt.client.widgets.events.ResizedEvent;
import com.smartgwt.client.widgets.events.ResizedHandler;
import com.smartgwt.client.widgets.layout.VLayout;

import de.intevation.flys.client.shared.model.BBox;
import de.intevation.flys.client.shared.model.MapInfo;

import org.gwtopenmaps.openlayers.client.Bounds;
import org.gwtopenmaps.openlayers.client.MapWidget;

/**
 * Panel that contains a MapWidget and a MapToolbar.
 * This panel is used by the flood map calculation input helper.
 */
public class MapPanel extends VLayout {

    protected MapToolbar      toolbar;

    protected FloodMap  floodMap;
    protected MapWidget floodMapWidget;
    protected boolean   digitizeEnabled;

    public MapPanel(MapInfo mapInfo, boolean digitizeEnabled) {
        BBox bbox = mapInfo.getBBox();

        this.digitizeEnabled = digitizeEnabled;
        this.floodMap        = new FloodMap(
            String.valueOf(mapInfo.getSrid()),
            new Bounds(
                bbox.getLowerX(),
                bbox.getLowerY(),
                bbox.getUpperX(),
                bbox.getUpperY()),
                640, 480);

        initLayout();
    }


    private void initLayout() {
        setWidth100();
        setHeight100();

        floodMapWidget = floodMap.getMapWidget();
        toolbar = new MapToolbar(floodMap, digitizeEnabled);

        addMember(toolbar);
        addMember(floodMapWidget);

        addResizedHandler(new ResizedHandler() {
            @Override
            public void onResized(ResizedEvent event) {
                doLayout();
            }
        });

        addParentMovedHandler(new ParentMovedHandler() {
            @Override
            public void onParentMoved(ParentMovedEvent event) {
                floodMapWidget.getMap().updateSize();
            }
        });
    }

    /**
     * Due to the fact that SmartGWT Layout misses a removeMember(Widget)
     * method we use this method to remove all WidgetCanvas members.
     * WidgetCanvas is the wrapper class used by Layout to handle Widget
     * objects.
     */
    protected void removeWidgetCanvasMember() {
        Canvas[] allCanvas = getMembers();
        for (Canvas canvas : allCanvas) {
            if (canvas instanceof WidgetCanvas) {
                removeMember(canvas);
            }
        }
    }

    protected void doLayout() {
        int width = getWidth();
        int height = getHeight();
        GWT.log("MapPanel.size: " + width + "x" + height);

        width -= 2; // minus black borders
        height -= toolbar.getHeight() + 4;

        // Recreate Map widget
        /*removeWidgetCanvasMember();
        floodMap.recreateWidget(width, height);
        floodMapWidget = floodMap.getMapWidget();
        addMember(floodMapWidget);
*/
        floodMapWidget.setSize(Integer.toString(width), Integer.toString(height));
        floodMapWidget.getMap().updateSize();
    }


    public FloodMap getFloodMap() {
        return floodMap;
    }

    public MapToolbar getMapToolbar () {
        return toolbar;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
